/* $Id: wwsr.h 5 2009-01-23 11:44:18Z gbv $ */
/***************************************************************************
 *                                                                         *
 *   wwsr - Wireless Weather Station Reader                                *
 *   2007 dec 19, Michael Pendec (michael.pendec@gmail.com)                *
 *   Version 0.1                                                           *
 *   2008 jan 24 Svend Skafte (svend@skafte.net)                           *
 *   Modifications for different firmware version(?)                       *
 *   2009 Jan 21 Guillermo Ballester gbv@oxixares.com                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*! \file metcal.h
    \brief include file for mectal library
*/

#ifndef METCAL_H
#define METCAL_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <signal.h>
#include <stdint.h>
#include <ctype.h>
#include <time.h>
#include <math.h>
#include <libintl.h>

#define MAX_CAL_POINTS 16

/*! \struct cal_point
   \brief A pair of points in a calibration, plus a spline interpolation coeficient  
*/
struct cal_point {
   double read; /*!< The value readed by instrument */
   double trusted; /*!< The true value it should readed in a perfect instrument */ 
   double spline; /*!< Spline coeficient, to be computed by spline interpolation routines */
};

/*!
  \struct cal_pol
  \brief An array of struct \ref cal_point for a variable of a instrument 
  It is the struct used to obtain corrected values from measured data
*/
struct cal_pol {
   int np; /*!< the number of struct \ref cal_ponit */
   struct cal_point p[MAX_CAL_POINTS]; /*!< the array of \ref cal_points . The elements are sorted by read value */
};

/*!
  \struct cal_prefix 
  \brief It associate a string prefix in files with calibration data with a struct \ref cal_pol 
*/ 
struct cal_prefix {
  char prefix[32]; /*!< the prefix string at the begining of a line in a calibration file */
  struct cal_pol *c; /*!< the pointer to a struct \ref cal_pol where to set the points with this prefix */
};

/*!
  \fn struct station_cal 
  \brief it stores the structure of calibration data of a station, for more than one sensor/parameters
*/
struct station_cal {
  int npf; /*!< number of structs \ref cal_prefix in the array */
  struct cal_prefix pf[32]; /*!< array of structs \ref cal_prefix */
};

void clean_cal_pol(struct cal_pol *c);
int read_cal_points(struct station_cal *s, const char *file);
double calibrated( double val, struct cal_pol *c);
int metcal_spline(struct cal_pol *c);
int spline_val(double *y, double x, struct cal_pol *c);
double es(double T);
double td( double t, double hr);
double wind_chill(double t, double v);
double heat_index( double t, double td);






#endif