/* $Id: metdb.c 19 2009-02-01 19:07:45Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file metdb.c
  \brief file with interface routines to deal with Berkeley DB
*/
# include "metdb.h"

int open_env(DB_ENV **env, char *envd)
{
  int ret;
  uint32_t env_flags;

  ret = db_env_create(env, 0);
  if (ret != 0)
    {
      fprintf(stderr, gettext("Error creating env handle: %s\n"), db_strerror(ret));
      return -1;
    }

  /* Open the environment. */
  env_flags = DB_CREATE |    /* If the environment does not exist,
                            * create it. */
              DB_INIT_MPOOL; /* Initialize the in-memory cache. */

  ret = (*env)->open(*env,   /* DB_ENV ptr */
                  envd,      /* env home directory */
                  env_flags,               /* Open flags */
                  0);                      /* File mode (default) */

  if (ret != 0)
    {
      fprintf(stderr, gettext("Environment open failed: %s\n"), db_strerror(ret));
      return -1;
    }

  return 0;
}

int open_db( DB **dbp, DB_ENV **env, uint32_t flags, char *envp, char *db)
{
  int ret;

  /* open the environment if any */
  if ( env != NULL && envp != NULL && strlen(envp) && open_env(env, envp))
    return ret;

  /* open the database */
  /* Initialize the structure. */
  if (env != NULL)
    ret = db_create(dbp, *env, 0);
  else
    ret = db_create(dbp, NULL, 0);
  if (ret != 0)
    {
      /* Error handling goes here */
      fprintf(stderr, gettext("Cannot init DB structure\n"));
      return -1;
    }

  /* open the database */
  ret = (*dbp)->open(*dbp,        /* DB structure pointer */
                  NULL,       /* Transaction pointer */
                  db, /* On-disk file that holds the database. */
                  NULL,       /* Optional logical database name */
                  DB_BTREE,   /* Database access method */
                  flags,      /* Open flags */
                  0);         /* File mode (using defaults) */
  if (ret != 0)
    {
      (*dbp)->err(*dbp, ret, gettext("Database open failed: %s"), db);
      return(ret);
    }
  return ret;
}

int close_db(DB *dbp, DB_ENV *env)
{
  /*
  * Close the database and environment
  */
  if (dbp != NULL)
    {
      dbp->close(dbp, 0);
    }
  if (env != NULL)
    {
      env->close(env, 0);
    }
}

int put_record_db(DBT *key, DBT *data, DB *dbp, uint32_t flags)
{
   return dbp->put(dbp, NULL, key, data, flags);
}

