/* $Id: ht71.c 30 2009-03-09 06:06:14Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 **************************************************************************/
#include "ht71.h"

int DT, SQL, NOCAL;
int DTEM, DHR;
char ENTRADA[256];
char SALIDA[256];
char INDIC[32];
char INICIO[16];
char FINAL[16];
time_t TINI, TFIN;
FILE *F, *O;
char SELF[] = "ht71";

#ifdef WITH_DB
int OVERWRITE, FROMDB, KILLDATA;
char DBFILE[PATH_LEN];
size_t INSERTED, REJECTED, ERRORS;
DB *HT71_DB;
struct obs_ht71 OBS_HT71[OBSLIMIT];
#endif

char CORFILE[PATH_LEN];


int main(int argc, char *argv[])
{
  int no;
  size_t nb;
  uint32_t i, dprog, dact, dpas;
  int16_t anyo, dt[2];
  double t, hr;
  char sensor[32], aux[64], fechai[32], fecha[32];
  uint8_t aux2[64];
  time_t tit;
  struct tm tim;
  struct obs_ht71 obs;

  /* lee argumentos de línea de comandos */
  if (lee_argumentos (argc, argv))
    exit (EXIT_FAILURE);

#ifdef WITH_DB
  if (FROMDB)
    {
      if (KILLDATA == 0)
        {
          if (open_db(&HT71_DB, NULL, DB_RDONLY, NULL, DBFILE))
            {
              return EXIT_FAILURE;
            }
        }
      else
        {
          if (open_db(&HT71_DB, NULL, 0, NULL, DBFILE))
            {
              return EXIT_FAILURE;
            }
        }
      goto inputdb;
    }

  /* open database to insert data*/
  if (strlen(DBFILE) && FROMDB == 0)
    {
      if (open_db(&HT71_DB, NULL, DB_CREATE, NULL, DBFILE))
        {
          return EXIT_FAILURE;
        }
    }
#endif


  /* Lee los primeros 7 bytes */
  if ((nb = fread(aux, 1, 7, F)) != 7)
    {
      fprintf(stderr, "Problemas al leer los 7 bytes iniciales");
      exit (EXIT_FAILURE);
    }

  /* lee el numero de datos programados */
  if ((nb = fread(&dprog, 4, 1, F)) != 1)
    {
      fprintf(stderr, "Problemas al leer el numero de datos programados");
      exit (EXIT_FAILURE);
    }
  else if (SQL == 0)
    fprintf(O, "# Numero de datos programados: %d\n", dprog);

  /* lee el numero de datos bajados */
  if ((nb = fread(&dact, 4, 1, F)) != 1)
    {
      fprintf(stderr, "Problemas al leer el numero de datos bajados");
      exit (EXIT_FAILURE);
    }
  else if (SQL == 0)
    fprintf(O, "# Numero de datos bajados    : %d\n", dact);

  /* lee el intervalo de datos */
  if ((nb = fread(&dpas, 4, 1, F)) != 1)
    {
      fprintf(stderr, "Problemas al leer el intervalo entre datos");
      exit (EXIT_FAILURE);
    }
  else if (SQL == 0)
    fprintf(O, "# Segundos entre datos       : %d\n", dpas);

  /* año, 2 bytes */
  if ((nb = fread(&anyo, 2, 1, F)) != 1)
    {
      fprintf(stderr, "Problemas al leer el año de inicio\n");
    }

  /* Lee 10 bytes */
  if ((nb = fread(aux, 1, 10, F)) != 10)
    {
      fprintf(stderr, "Problemas al leer los 7 bytes iniciales");
      exit (EXIT_FAILURE);
    }

  /* lee mes, dia, hora, minuto y segundo de inicio */
  if ((nb = fread(aux2, 1, 5, F)) != 5)
    {
      fprintf(stderr, "Problemas al leer la fecha");
      exit (EXIT_FAILURE);
    }
  else if (SQL == 0)
    printf("# Inicio %d-%02i-%02i %02i:%02i:%02i\n", anyo, aux2[0], aux2[1], aux2[2], aux2[3], aux2[4]);

  /* lee 2 bytes */
  if ((nb = fread(aux, 1, 2, F)) != 2)
    {
      fprintf(stderr, "Problemas al leer bytes 36-37\n");
      exit (EXIT_FAILURE);
    }

  /* lee cadena sensor */
  if ((nb = fread(sensor, 1, 13, F)) != 13)
    {
      fprintf(stderr, "Problemas al leer el nombre\n");
      exit (EXIT_FAILURE);
    }
  else if (SQL == 0)
    printf("# Nombre '%s'\n",sensor);

  /* lee 16 bytes */
  if ((nb = fread(aux, 1, 16, F)) != 16)
    {
      fprintf(stderr, "Problemas al leer bytes 51-66\n");
      exit (EXIT_FAILURE);
    }

  /* Escribe cadena con hora inicial */
  sprintf(fechai, "%04d%02i%02i%02i%02i%02i", anyo, aux2[0], aux2[1], aux2[2], aux2[3], aux2[4]);
  memset(&tim, 0, sizeof(struct tm));
  strptime(fechai, "%Y%m%d%H%M%S", &tim);
  tit = timegm(&tim) - DT;

  /* ahora los datos */
  for (i = 0; i < dact; i++, tit += dpas)
    {
      if ((nb = fread(dt, 2, 2, F)) != 2)
        {
          fprintf(stderr, "Problemas al leer el dato %d\n",i);
          exit (EXIT_FAILURE);
        }

      /* comprueba si es un dato válido a considerar */
      if (strlen(INICIO) && tit < TINI)
        continue;
      if (strlen(FINAL) && tit > TFIN)
        continue;

      /* pasa los datos a coma ºC y % */
      t = value((double)dt[0] * 0.1, &CALt);
      hr = value((double)dt[1] * 0.1, &CALh);
      gmtime_r (&tit, &tim);
      if (SQL)
        {
          strftime(fecha, 32, "%Y%m%d%H%M%S",&tim);
          fprintf(O,"%s\t%s\t%.1lf\t%.1lf\n",INDIC, fecha, t, hr);
        }
      else
        {
          strftime(fecha, 32,"%Y-%m-%d %H:%M:%S UTC", &tim);
          fprintf(O,"%s: %5.1lf C %5.1lf %%\n", fecha, t, hr);
        }
#ifdef WITH_DB
      if (strlen(DBFILE))
        {
          obs.datime = tit;
          memcpy(&obs.dat[0], &dt[0], 4);
          insert_db_obs_ht71(HT71_DB, &obs);
        }
#endif
    }

  fclose(F);
#ifdef WITH_DB
  if (strlen(DBFILE))
    {
      close_db(HT71_DB, NULL);
      fprintf(stdout, gettext("Base datos: Insertados %lu, rechazados %lu, errores %lu\n"), INSERTED, REJECTED, ERRORS);
    }
#endif

  return EXIT_SUCCESS;

#ifdef WITH_DB
inputdb:

  /* lee los datos de la BD */
  no = get_db_obs_ht71( HT71_DB , &OBS_HT71[0], OBSLIMIT, TINI, TFIN);
  close_db(HT71_DB, NULL);

  for (i = 0; i < no; i++)
    {
      /* pasa los datos a coma ºC y % */
      t = value(0.1 * (double)(OBS_HT71[i].dat[0]), &CALt);
      hr = value(0.1 * (double)(OBS_HT71[i].dat[1]), &CALh);
      gmtime_r (&OBS_HT71[i].datime, &tim);
      if (SQL)
        {
          strftime(fecha, 32, "%Y%m%d%H%M%S",&tim);
          fprintf(O,"%s\t%s\t%.1lf\t%.1lf\n",INDIC, fecha, t, hr);
        }
      else
        {
          strftime(fecha, 32,"%Y-%m-%d %H:%M:%S UTC", &tim);
          fprintf(O,"%s: %5.1lf C %5.1lf %%\n", fecha, t, hr);
        }

    }
  return EXIT_SUCCESS;

#endif
}
