/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor   *
 *   gbv@oxixares.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ht71.h"

#ifdef WITH_DB
/*
   Try to insert a record to database
*/
int insert_db_obs_ht71(DB *dbp, struct obs_ht71 *obs)
{
  int ret;
  uint32_t flag;
  DBT key, data;
  char aux[16];
  struct tm tim;

  /* clear key and data */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  gmtime_r(&obs->datime, &tim);
  strftime(aux,16, "%Y%m%d%H%M%S", &tim);

  key.data = aux;
  key.size = strlen(aux) + 1;
  data.data = obs->dat;
  data.size = 4;

  if (OVERWRITE)
    {
      flag = 0;
    }
  else
    flag = DB_NOOVERWRITE;

  ret = put_record_db(&key, &data, dbp, flag);
  if (ret == 0)
    INSERTED++;
  else if (ret == DB_KEYEXIST)
    REJECTED++;
  else
    ERRORS++;
}

/*
   return the Obs readed
*/
int get_db_obs_ht71(DB *dbp, struct obs_ht71 *obs, size_t maxobs, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M%S", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M%S", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /*flag for first time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0 && no < maxobs)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M%S", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          memcpy(&obs[no].dat[0], data.data, 4);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}

/*
   return the Obs deleted
*/
int del_db_obs_ht71(DB *dbp, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M%S", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M%S", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /*flag for first time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M%S", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          if (curs->c_del(curs, 0) == 0)
            no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}

#endif