/* $Id: ht71_io.c 30 2009-03-09 06:06:14Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ht71.h"

void print_usage(void)
{
  printf("Uso:\n   pceht71 [-h] -f fichero_rec [-o fichero_out] [-i indicativo][-U ds]\n");
  printf("    [-I inicio][-F final][-s sensor][-q][-H cor_hum][-T cor_temp]\n");
  printf("    -h. Imprime esta ayuda\n");
#ifdef WITH_DB
  printf("    -d Los datos se leen de un fichero base de datos (ver la opción -D)\n");
  printf("    -D Database_file. Fichero de base de datos.\n");
  printf("    -K Borra los datos del intervalo seleccionado en la Base de datos\n");
#endif
  printf("    -C cor_file. Fichero de corrección de temperatura y humedad relativa\n");
  printf("    -f fichero_rec. Fichero de entrada tipo rec. Debe especificar el nombre completo\n");
  printf("    -F Hora UTC del final del intervalo de datos a considerar en formato AAAAMMDDHHMMSS.\n");
  printf("       Por defecto hasta el final del fichero\n");
  printf("    -i indicativo. Indicativo de la estación. Por defecto 'PCH-HT71'. Max 31 caracteres.\n");
  printf("    -I Hora UTC de inicio del intervalo de datos a considerar en formato AAAAMMDDHHMMSS.\n");
  printf("       Por defecto desde el inicio del fichero\n");
  printf("    -q Formato preparado para base de datos (separado por tabuladores)\n");
  printf("    -o fichero_out. Fichero de salida. Por defecto a stdout.\n");
  printf("    -U ds. Segundos de desviación de la hora del datalogger respecto a UTC. Por defecto 0\n");
  printf("       Si el datalogger esta adelantado con respecto UTC, ds es positivo, en otro caso el\n");
  printf("       argumento negativo debe encerrarse entre comillas.\n");
}

/*
  Lee los argumentos de la línea de comandos
*/
int lee_argumentos( int _argc, char * _argv[])
{
  int iopt, j;
  size_t l;
  struct tm taux;

  /* Valores por defecto */
  DT = 0;
  DTEM = 0;
  DHR = 0;
  SQL = 0;
  O = stdout;
  F = NULL;
  strcpy(INDIC,"PCE-HT71");
  INICIO[0] = '\0';
  FINAL[0] = '\0';
  CORFILE[0] = '\0';
  NOCAL = 0;
#ifdef WITH_DB
  DBFILE[0] = '\0';
  FROMDB = 0;
  INSERTED = 0;
  REJECTED = 0;
  ERRORS = 0;
  KILLDATA = 0;
#endif

  /*
     Lee las opciones de entrada
  */
  while ((iopt = getopt(_argc, _argv, "hC:dD:i:I:Kf:F:o:qU:"))!=-1)
    switch (iopt)
      {
      case 'f':
        if (strlen(optarg) > 0)
          {
            strcpy(ENTRADA, optarg);
            if ((F = fopen(ENTRADA, "r")) == NULL)
              {
                fprintf(stderr, "No se puede abrir %s\n", ENTRADA);
                exit (EXIT_FAILURE);
              }
          }
        else
          {
            fprintf(stderr, "Debe indicar un fichero de entrada\n");
            print_usage();
            exit (EXIT_FAILURE);
          }
        break;
      case 'o':
        if (strlen(optarg) > 0)
          {
            strcpy(SALIDA, optarg);
            if ((F = fopen(SALIDA, "w")) == NULL)
              {
                fprintf(stderr, "No se puede abrir %s\n", SALIDA);
                exit (EXIT_FAILURE);
              }
          }
        break;
      case 'q':
        SQL = 1;
        break;
      case 'U':
        if ((l = strlen(optarg)) > 0)
          {
            if (strspn(optarg,"-+0123456789") == l)
              DT = atoi(optarg);
            else
              {
                fprintf(stderr, "Número de segundos %s respecto UTC erróneo\n", optarg);
                exit (EXIT_FAILURE);
              }
          }
        break;
      case 'C':
        if ((l = strlen(optarg)) > 0)
          {
             strcpy(CORFILE, optarg);
          }
        break;
#ifdef WITH_DB
        case 'd':
          FROMDB = 1;
          break;
        case 'D':
          if (strlen(optarg))
          {
             strcpy(DBFILE, optarg);
          }
          break;
        case 'K':
          KILLDATA = 1;
          break;
#endif
      case 'I':
        if (strlen(optarg) != 14)
          {
             fprintf(stderr, "La fecha de inicio debe ir en formato AAAAMMDDHHmmss\n");
             exit(EXIT_FAILURE);
          }
        strcpy(INICIO, optarg);
        break;
      case 'F':
        if (strlen(optarg) != 14)
          {
             fprintf(stderr, "La fecha de final debe ir en formato AAAAMMDDHHmmss\n");
             exit(EXIT_FAILURE);
          }
        strcpy(FINAL, optarg);
        break;
      case 'i':
        if (strlen(optarg) == 0 || strlen(optarg) >= 32)
        {
             fprintf(stderr, "El indicativo solamente puede tener hasta 31 caracteres\n");
             exit(EXIT_FAILURE);
        }
        strcpy(INDIC, optarg);
        break;
      default:
        print_usage();
        exit (EXIT_FAILURE);
      }

#ifdef WITH_DB
  if (F == NULL && FROMDB == 0)
    {
      fprintf(stderr, "Debe indicar un fichero *.rec de entrada o una base de datos\n");
      print_usage();
      exit (EXIT_FAILURE);
    }

  if (FROMDB  && strlen(DBFILE) == 0)
    {
      fprintf(stderr, "Debe indicar un fichero de base de datos de donde leer los datos\n");
      print_usage();
      exit (EXIT_FAILURE);
    }

  if (KILLDATA && (FROMDB == 0 || strlen(DBFILE) == 0))
  {
     fprintf(stderr, "Debe indicar la base de datos con '-D' y utilizar la opción '-d' junto con '-K'\n");
     exit (EXIT_FAILURE); 
  }
#endif

  /* calcula instantes inicial y final */
  if (strlen(INICIO))
  {
     memset(&taux, 0, sizeof(struct tm));
     strptime(INICIO, "%Y%m%d%H%M%S", &taux);
     TINI = timegm(&taux);
  }

  if (strlen(FINAL))
  {
     memset(&taux, 0, sizeof(struct tm));
     strptime(FINAL, "%Y%m%d%H%M%S", &taux);
     TFIN = timegm(&taux);
  }

  /* lee calibrados */
  if (strlen(CORFILE))
  {
     if (read_cal_points(&CALHT71, CORFILE))
     {
        fprintf(stderr, "Problems when reading calibration values from file\n");
        return 1;
     }

  }
  else
  {
     NOCAL = 1;
     for (j = 0; j < CALHT71.npf; j++)
       clean_cal_pol(CALHT71.pf[j].c);
  }
  return 0;
}
