/* $Id: wwsr3.c 38 2009-04-25 21:35:47Z gbv $ */
/***************************************************************************
 *   wwsr - Wireless Weather Station Reader                                *
 *   2007 dec 19, Copyright (C) 2007 by Michael Pendec                     *
 *     michael.pendec@gmail.com                                            *
 *   Version 0.1                                                           *
 *                                                                         *
 *   2008 jan 24 , Copyright (C) 2008 Svend Skafte                         *
 *     svend@skafte.net                                                    *
 *   Modifications for different firmware version(?)                       *
 *                                                                         *
 *   2009 Jan 21 Guillermo Ballester                                       *
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *     Calibration, Database, gettext ...                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


/*
 * wwsr - Wireless Weather Station Reader
 * 2007 dec 19, Michael Pendec (michael.pendec@gmail.com)
 * Version 0.1
 * 2008 jan 24 Svend Skafte (svend@skafte.net)
 * Modifications for different firmware version(?)
 * 2009 Jan 21 Guillermo Ballester
 */
#include "wwsr.h"


struct usb_dev_handle *devh;
int VIEWMASK;
int ret, resetws, pdebug;
uint8_t LOGBUF[MEMSIZE], CURBUF[32];
char usbmsg[16], *endptr;
struct  obs_data Obs[4096];
struct extrem_data hin_max, hin_min, tin_max, tin_min, hout_max, hout_min, tout_max, tout_min, chill_max,
      chill_min, dwp_max, dwp_min, apres_max, apres_min, rpres_max, rpres_min, wsp_max, gust_max,
      rh_max, rd_max, rw_max, rm_max, rt_max;
time_t TBEGIN, TEND;
int ONLYLAST, FROMFILE, EXTREM, LASTN, GMTOFF, TIMESET, FROMDB, KILLDATA;
char BACKUPFILE[PATH_LEN], INPUTFILE[PATH_LEN], CALFILE[PATH_LEN];
char DATEBEGIN[16], DATEEND[16];
FILE *F;
char wd[17][4]={"N","NNE","NE","ENE","E","ESE","SE","SSE","S","SSW","SW","WSW","W","WNW","NW","NNW","--"};

#ifdef WITH_DB
int OVERWRITE, FROMDB, CLIMAT, INTERP;
char DBFILE[PATH_LEN];
size_t INSERTED, REJECTED, ERRORS;
DB *WWSR_DB;
struct stat_summary Summary;
#endif


int main(int argc, char **argv)
{
  unsigned int nobs, npar;
  size_t nb = 0;
  int i, j;
  uint16_t  buftemp, current;
  char aux[32];
  time_t ti;
  struct tm tim;
#ifdef WITH_DB
  struct real_data r;
#endif
  /* gettext stuf */
  setlocale (LC_ALL, "");
  /*setlocale (LC_MESSAGES, "es_ES.utf8");*/
  textdomain (PACKAGE);
  bindtextdomain (PACKAGE, LOCALEDIR);

  /* read the arguments */
  if ( read_arguments(argc, argv))
    exit (EXIT_FAILURE);

  if (FROMFILE)
    {
      if ((F = fopen(INPUTFILE, "r")) == NULL)
        {
          fprintf(stderr, gettext("Cannot open the input file %s\n"), INPUTFILE);
          return EXIT_FAILURE;
        }
    }
#ifdef WITH_DB
  else if (FROMDB)
    {
      if (KILLDATA == 0)
        {
          if (open_db(&WWSR_DB, NULL, DB_RDONLY, NULL, DBFILE))
            {
              if (FROMFILE == 0)
                _close_readw();
              return EXIT_FAILURE;
            }
        }
      else
        {
          if (open_db(&WWSR_DB, NULL, 0, NULL, DBFILE))
            {
              if (FROMFILE == 0)
                _close_readw();
              return EXIT_FAILURE;
            }
        }

      goto inputdb;
    }
#endif
  else
    {
      /* open the usb device */
      _open_readw();

      /* init */
      _init_wread();
    }

  /* reset LOGBUF */
  memset(LOGBUF, 0, MEMSIZE);

  /* If not only last, then read all the memory from station */
  if (ONLYLAST == 0)
    {
      /* read the data */
      nb = read_station_buf(LOGBUF, MEMSIZE);
    }
  else
    {
      get_main_block((char *)LOGBUF);
    }

  /* backup file if any */
  if (FROMFILE == 0 &&
      FROMDB == 0 &&
      ONLYLAST == 0 &&
      strlen(BACKUPFILE))
    {
      if (write_backup_file(BACKUPFILE, nb))
        {
          _close_readw();
          return EXIT_FAILURE;
        }
    }

  /* check if want to reset history */
  if (FROMFILE == 0 && resetws == 1)
    {
      reset_ws();
      _close_readw();
      return EXIT_SUCCESS;
    }

  /* current observation position in memory */
  current = read_uint16(LOGBUF + 30);

  /* Computes the unix time when reading data*/
  read_date(&tim, LOGBUF + 43);
  ti = timegm(&tim) - GMTOFF;

#ifdef WITH_DB
  /* open database */
  if (strlen(DBFILE) && FROMDB == 0)
    {
      if (open_db(&WWSR_DB, NULL, DB_CREATE, NULL, DBFILE))
        {
          if (FROMFILE == 0)
            _close_readw();
          return EXIT_FAILURE;
        }
    }
#endif

  if (ONLYLAST && FROMDB == 0)
    {
      if (current > 256)
        {
          current -= 16;
          read_current( LOGBUF, current);
          buf2obs(&(Obs[0]), LOGBUF + current + 16);
        }
      else
        {
          read_current( LOGBUF, current);
          buf2obs(&(Obs[0]), LOGBUF + current);
        }
      Obs[0].datime = ti;
      print_obs(stdout, &(Obs[0]), "\t");
#ifdef WITH_DB
      if (strlen(DBFILE))
        {
          insert_db_obs( WWSR_DB, &Obs[0]);
          if (INSERTED)
            fprintf(stdout, gettext("Inserted into Database\n"));
          else if (REJECTED)
            fprintf(stdout, gettext("Rejected. Observation already in Database\n"));
          else
            fprintf(stdout, gettext("Error trying to introduce the data into Database\n"));
          close_db (WWSR_DB, NULL);
        }
#endif
      if (FROMFILE)
        fclose(F);
      else
        _close_readw();
      return 0;
    }

  if (LASTN == 0)
    {
      nobs = 4080;
      printf(gettext("# Reading %d observations\n"), nobs);
    }
  else
    {
      printf(gettext("# Reading latest %d observations\n"), LASTN);
      nobs = LASTN + 1;
    }


  strftime(aux, 32, "%Y-%m-%d %H:%M", &tim);
  printf(gettext("# Station date /time: %s\n"), aux);

  /* read all the data */
  for ( j = nobs - 1, i = 0; i < nobs ; i++, j--)
    {
      buftemp = get_pos(current, j);
      buf2obs(&(Obs[i]), LOGBUF + buftemp);
    }

  /*
    we know what minute is current reading. Now we have to subtract the interval member of every
    observation to get the proper time
  */
  /* computes date/time */

  Obs[nobs - 1].datime = ti;
  for (i = nobs - 1; i > 0; i--)
    {
      if (Obs[i].mask == 0xFF)
        break;
      ti -= (Obs[i].interval * 60);
      Obs[i - 1].datime = ti;
    }

  /* print obs */
  for (i = 0; i < nobs - 1; i++)
    {
      print_obs(stdout, &(Obs[i]), "\t");
    }

#ifdef WITH_DB
  if (strlen(DBFILE))
    {
      for (i = 0; i < nobs - 1; i++)
        {
          /* check about date */
          if ( (TBEGIN == 0 || Obs[i].datime >= TBEGIN) &&
               (TEND == 0 || Obs[i].datime <= TEND))
            {
              insert_db_obs(WWSR_DB, &Obs[i]);
            }
        }
      close_db(WWSR_DB, NULL);
      fprintf(stdout, gettext("# Database: Inserted %u, rejected %u, errors %u\n"), (unsigned int)INSERTED,
              (unsigned int)REJECTED, (unsigned int)ERRORS);
    }

#endif

  fprintf(stdout, gettext("\n# Current reading:\n"));
  print_obs(stdout, &(Obs[nobs - 1]), "\t");


  if (EXTREM)
    {
      read_extremes(LOGBUF);
      print_all_extremes(stdout);
    }


  if (FROMFILE)
    fclose(F);
  else
    _close_readw();
  return 0;

#ifdef WITH_DB
inputdb:
  if (KILLDATA)
    {
      nobs = del_db_obs(WWSR_DB, 0x00, TBEGIN, TEND);
      fprintf(stdout, gettext("# Database: Deleted %u\n"), (unsigned int)nobs);
      close_db(WWSR_DB, NULL);
      return 0;
    }

  /* clean if needed */
  if (CLIMAT)
  {
    memset((void *)&Summary, 0, sizeof(struct stat_summary));
    Summary.begin = TBEGIN;
    Summary.end = TEND;
  }

  /* The reading task is in blocks of 4096 data */
  ti = TBEGIN;
  nobs = 0;
  while ((npar = get_db_obs_wide(WWSR_DB, &Obs[0], 4096, ti, TEND)) == 4096)
    {
      nobs += npar;
      ti = Obs[4095].datime + Obs[4095].interval * 60; //assumed next obs interval is the same
      for (i = 0; i < npar; i++)
        {
          if (CLIMAT)
            {
              // first obs may be prior to tbegin, we then
              // substitute it by an interpolated real_data
              // Same for last obs in the wide interval
              if (Obs[i].datime < TBEGIN)
                {
                  // It is usually first obs
                  interp_data(&r, &Obs[i], 2, TBEGIN);
                }
              else if (Obs[i].datime > TEND)
                {
                  // it is last obs
                  interp_data(&r, &Obs[i - 1], 2, TEND);
                }
              else
                {
                  obsdata_to_realdata( &r, &(Obs[i]));
                }
              summarize_realdata(&Summary, &r);
            }
          else
            print_obs_nores(stdout, &(Obs[i]), "\t");
        }
    }

  // last 4096 block
  nobs += npar;
  close_db(WWSR_DB, NULL);

  /* print obs */
  if (INTERP == 0)
    {
      for (i = 0; i < npar; i++)
        {
          if (CLIMAT)
            {
              // first obs may be prior to tbegin, we then
              // substitute it by an interpolated real_data
              // Same for last obs in the wide interval
              if (Obs[i].datime < TBEGIN)
                {
                  // It is usually first obs
                  interp_data(&r, &Obs[i], 2, TBEGIN);
                }
              else if (Obs[i].datime > TEND)
                {
                  // it is last obs
                  interp_data(&r, &Obs[i - 1], 2, TEND);
                }
              else
                {
                  obsdata_to_realdata( &r, &(Obs[i]));
                }
              summarize_realdata(&Summary, &r);
            }
          else
            print_obs_nores(stdout, &(Obs[i]), "\t");
        }
    }
  else
    {
      interp_data (&r, &Obs[0], nobs, TBEGIN);
      print_real_data(stdout, &r, "\t");
    }

  if (CLIMAT)
    print_stat_summary(&Summary, stdout);

  return 0;
#endif
}
