/* $Id: wwsr_db.c 38 2009-04-25 21:35:47Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file wwsr_db.c
  \brief file with support for Berkeley DB
*/
#include "wwsr.h"

#ifdef WITH_DB
/*
   Try to insert a record to database
*/
int insert_db_obs(DB *dbp, struct obs_data *obs)
{
  int ret, nobs, i;
  uint32_t flag;
  DBT key, data;
  char aux[16];
  struct obs_data obx[3];
  struct tm tim;

  /* check if a valid observadion */
  if ( obs->buf[15] == 0xFF || (obs->buf[0] == 0 && obs->buf[1] == 0))
    {
      REJECTED++;
      return 0;
    }

  /* clear key and data */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  gmtime_r(&obs->datime, &tim);
  strftime(aux,16, "%Y%m%d%H%M", &tim);

  key.data = aux;
  key.size = strlen(aux) + 1;
  data.data = obs->buf;
  data.size = 16;

  if (OVERWRITE)
    {
      flag = 0;
    }
  else
    flag = DB_NOOVERWRITE;

  if ((obs->mask & 0x01) == 0)
    {
      /*
        Check if there are bad overlapped time for non-current observations
        Two observations not marked as current (mask == 0x01) cannot be spaced less than
        two minutes
      */
      nobs = get_db_obs(dbp, &obx[0], 3, obs->datime - 60, obs->datime + 60);
      if (nobs)
        {
          for (i = 0; i < nobs; i++)
            {
              if ((obx[i].mask & 0x01) == 0)
                {
                  /* found overlap, reject */
                  REJECTED++;
                  return 0;
                }
            }
        }
    }


  ret = put_record_db(&key, &data, dbp, flag);
  if (ret == 0)
    INSERTED++;
  else if (ret == DB_KEYEXIST)
    REJECTED++;
  else
    ERRORS++;
  return 0;
}

/*
   return the Obs readed
*/
int get_db_obs(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0 && no < maxobs)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}

/*
   return the Obs readed
   This version includes previous record to tbegin and/or next record to tend if the time
  does not mtaches
*/
int get_db_obs_wide(DB *dbp, struct obs_data *obs, size_t maxobs, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */

  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      // first valor equal or after datei
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (ti == tbegin)
        {
          // this record is valid
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
          flag = DB_NEXT;
        }
      else
        flag = DB_PREV; // we first will load the previous
    }
  else
    {
      if (curs != NULL)
        curs->c_close(curs);
      return no;
    }

  /* First try to read a data */
  if ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
  {
    memset(&tim, 0, sizeof(struct tm));
    strptime( key.data, "%Y%m%d%H%M", &tim);
    ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (obs[no - 1].datime < tend)
          {   
            buf2obs( &obs[no], data.data);
            obs[no].datime = ti;
            no++;
          }
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
  }
  else if (ret == DB_NOTFOUND)
  {
     flag = DB_NEXT;
  }

  /* And now a loop for the next records */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0 && no < maxobs)
    {
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs[no], data.data);
          obs[no].datime = ti;
          no++;
        }
      else
        {
          /* we surpassed the end of interval */
          if (obs[no - 1].datime < tend)
          {   
            buf2obs( &obs[no], data.data);
            obs[no].datime = ti;
            no++;
          }
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}



/*
   return the Obs deleted
*/
int del_db_obs(DB *dbp, uint8_t mask, time_t tbegin, time_t tend)
{
  int no, ret;
  char datei[16], datef[16];
  uint32_t flag;
  time_t ti;
  struct tm tim;
  struct obs_data obs;
  DBC *curs;
  DBT key, data;

  /* set a cursor */
  dbp->cursor( dbp, NULL, &curs, 0);

  /* set first and last key from the interval*/
  memset(&tim, 0, sizeof(struct tm));
  ti = tbegin;
  gmtime_r(&ti, &tim);
  strftime(datei, 16, "%Y%m%d%H%M", &tim);
  ti = tend;
  gmtime_r(&ti, &tim);
  strftime(datef, 16, "%Y%m%d%H%M", &tim);

  /* set initial key, search */
  memset(&key, 0, sizeof (DBT));
  memset(&data, 0, sizeof (DBT));

  key.data = datei;
  key.size = strlen(datei) + 1;

  /* loop to search data */
  no = 0;
  flag = DB_SET_RANGE; /* flag for fisrt time */
  while ((ret = curs->c_get(curs, &key, &data, flag)) == 0)
    {
      /* first read */
      memset(&tim, 0, sizeof(struct tm));
      strptime( key.data, "%Y%m%d%H%M", &tim);
      ti = timegm(&tim);
      if (tend == 0 || ti <= tend)
        {
          buf2obs( &obs, data.data);
          /* if mask matches the delete */
          if (mask == obs.mask)
            {
              if (curs->c_del(curs, 0) == 0)
                no++;
            }
        }
      else
        {
          /* we surpassed the end of interval */
          if (curs != NULL)
            curs->c_close(curs);
          return no;
        }
      flag = DB_NEXT;
    }

  /* close the cursor in case of end or max memory*/
  if (curs != NULL)
    curs->c_close(curs);
  return no;
}


#endif
