/* $Id: wwsr_io.c 38 2009-04-25 21:35:47Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "wwsr.h"


void show_usage(void)
{
  printf("Wireless Weather Station Reader v0.3\n");
  printf("(C) 2007 Michael Pendec \n(C) 2009 Guillermo Ballester Valor\n\n");
  printf(gettext("Usage:\n"));
  printf(gettext("wwsr [options]\n\n"));
  printf(gettext("options\n"));
  printf(gettext(" -a       Show all parameters (overrides -V parameters). Is the default\n"));
  printf(gettext(" -b       Backup_file. File where stores binary content of station memory, as backup.\n"));
  printf(gettext("          Mandatory when reset log buffer with -z\n"));
  printf(gettext(" -c       Show just the current data. See -V option\n"));
  printf(gettext(" -B       Begin_date. Consider only data since Begin date, format YYYYMMDDHHmm\n"));
  printf(gettext("          Default is since the data begin in source\n"));
  printf(gettext(" -C       Cal_file. Calibration file. A calibration file is a plain text file with three\n"));
  printf(gettext("          items per line: prefix_string readed_value trusted_value\n"));
  printf(gettext("          A line with '#' at the begining is a comment. The prefix_string is associated to\n"));
  printf(gettext("          a parameter and is the same that the substring in -V argument, except 'd'\n"));
#ifdef WITH_DB
  printf(gettext(" -d       Data are read from database file (see -D argument)\n"));
  printf(gettext(" -D       database_file. File database.\n"));
  printf(gettext(" -K       Kill database interval. Must be used with -d -D options.\n"));
  printf(gettext(" -S       Get the climatological statistics in defined interval.\n"));
  printf(gettext(" -T       Get the interpolated values at date, format YYYYMMDDHHmm\n"));
#endif
  printf(gettext(" -E       End_date. Consider only data till END_date, format YYYYMMDDHHmm\n"));
  printf(gettext("          Default is till the end of log buffer in station. Not the current data\n"));
  printf(gettext(" -h       Help information. Show these lines\n"));
  printf(gettext(" -i       Input_file. Input from a binary backup instead of station\n"));
  printf(gettext(" -l       last_n. Show only last_n observations, not the current. Default is all history\n"));
  printf(gettext(" -x       Show bytes retrieved from device. For debug proposes\n"));
  printf(gettext(" -z       Reset log buffer (will ask for confirmation, backup must be active).\n"));
  printf(gettext("          This option is not recommended. Better reset history in the station.\n"));
  printf(gettext(" -U       gmtoff. Seconds to add to UTC time to obtain the panel time.\n"));
  printf(gettext(" -V       List_parameters. A string which substrings selects the parameters:\n"));
  printf(gettext("          'd' = minutes between data (log step interval)\n"));
  printf(gettext("          'ti' = internal temperature\n"));
  printf(gettext("          'to' = external temperature\n"));
  printf(gettext("          't' = temperature (both internal/external) \n"));
  printf(gettext("          'hi' = internal humidity\n"));
  printf(gettext("          'ho' = external humidity\n"));
  printf(gettext("          'h' = humidity (both internal/external)\n"));
  printf(gettext("          'p' = pressure\n"));
  printf(gettext("          'w' = wind\n"));
  printf(gettext("          'r' = rain\n"));
  printf(gettext(" -X       Show also extremes stored by station. Parameters showed are controlled by -V\n"));
  printf(gettext("          If dates are out of selected interval with -B and -E they are not showed\n"));
}

int read_arguments(int argc, char **argv)
{
  int c, j;
  struct tm taux;

  /* Default values */
  VIEWMASK = 0;
  BACKUPFILE[0] = '\0';
  INPUTFILE[0] = '\0';
  DATEBEGIN[0] = '\0';
  DATEEND[0] = '\0';
  CALFILE[0] = '\0';
#ifdef WITH_DB
  DBFILE[0] = '\0';
  FROMDB = 0;
  INSERTED = 0;
  REJECTED = 0;
  ERRORS = 0;
  KILLDATA = 0;
  CLIMAT = 0;
  INTERP = 0;
#endif
  ONLYLAST = 0;
  resetws = 0;
  FROMFILE = 0;
  TBEGIN = 0;
  TEND = 0;
  EXTREM = 0;
  NOCAL = 0;
  LASTN = 0;
  GMTOFF = 0;
  TIMESET = 0;
  
  while ((c = getopt (argc, argv, "ab:cB:C:dD:E:hi:Kl:ST:U:V:xzX")) != -1)
    {
      switch (c)
        {
        case 'a':
          VIEWMASK = MASK_ALL;
          break;
        case 'b':
          strcpy(BACKUPFILE, optarg);
          break;
        case 'c':
          ONLYLAST = 1;
          break;
        case 'B':
          if (strlen(optarg) != 12 || strspn(optarg, "0123456789") != 12)
            {
              fprintf(stderr, gettext("Begin date must be in YYYYMMDDHHmm format\n"));
              exit(EXIT_FAILURE);
            }
          strcpy(DATEBEGIN, optarg);
          memset(&taux, 0, sizeof(struct tm));
          strptime(DATEBEGIN, "%Y%m%d%H%M%S", &taux);
          TBEGIN = timegm(&taux);
          break;
        case 'C':
          if (strlen(optarg))
          {
             strcpy(CALFILE, optarg);
          }
          break;
#ifdef WITH_DB
        case 'd':
          FROMDB = 1;
          break;
        case 'D':
          if (strlen(optarg))
          {
             strcpy(DBFILE, optarg);
          }
          break;
        case 'K':
          KILLDATA = 1;
          break;
        case 'S':
          CLIMAT = 1;
          break;
        case 'T':
          if (strlen(optarg) != 12 || strspn(optarg, "0123456789") != 12)
            {
              fprintf(stderr, gettext("Date must be in YYYYMMDDHHmm format\n"));
              exit(EXIT_FAILURE);
            }
          strcpy(DATEEND, optarg);
          memset(&taux, 0, sizeof(struct tm));
          strptime(DATEEND, "%Y%m%d%H%M", &taux);
          TEND = timegm(&taux);
          strcpy(DATEBEGIN, DATEEND);
          TBEGIN = TEND;
          INTERP = 1;
          break;
#endif
        case 'E':
          if (strlen(optarg) != 12 || strspn(optarg, "0123456789") != 12)
            {
              fprintf(stderr, gettext("End date must be in YYYYMMDDHHmm format\n"));
              exit(EXIT_FAILURE);
            }
          strcpy(DATEEND, optarg);
          memset(&taux, 0, sizeof(struct tm));
          strptime(DATEEND, "%Y%m%d%H%M%S", &taux);
          TEND = timegm(&taux);
          break;
        case 'i':
          if (strlen(optarg))
          {
             strcpy(INPUTFILE, optarg);
             FROMFILE = 1;
          }
          break;
        case 'l':
          if (strlen(optarg) && (strspn(optarg,"0123456789") == strlen(optarg)))
          {
            LASTN = atoi(optarg);
          }
          break;
        case 'U':
          if (strlen(optarg) && (strspn(optarg,"0123456789") == strlen(optarg)))
          {
            GMTOFF = atoi(optarg);
            TIMESET = 1;
          }
          break;
        case 'V':
          if (strstr(optarg,"d") != NULL)
            VIEWMASK |= MASK_INTERV;
          if  (strstr(optarg,"ti") != NULL)
            VIEWMASK |= MASK_TEMP_INT;
          if (strstr(optarg,"to") != NULL)
            VIEWMASK |= MASK_TEMP_EXT;
          if ((VIEWMASK & MASK_TEMP_INT) == 0 &&
              (VIEWMASK & MASK_TEMP_EXT) == 0 &&
              strstr(optarg,"t") != NULL)
            VIEWMASK |= MASK_TEMP;

          if  (strstr(optarg,"hi") != NULL)
            VIEWMASK |= MASK_HUM_INT;
          if (strstr(optarg,"ho") != NULL)
            VIEWMASK |= MASK_HUM_EXT;
          if ((VIEWMASK & MASK_HUM_INT) == 0 &&
              (VIEWMASK & MASK_HUM_EXT) == 0 &&
              strstr(optarg,"h") != NULL)
            VIEWMASK |= MASK_HUM;

          if (strstr(optarg,"p") != NULL)
            VIEWMASK |= MASK_PRES;

          if (strstr(optarg,"w") != NULL)
            VIEWMASK |= MASK_WIND;

          if (strstr(optarg,"r") != NULL)
            VIEWMASK |= MASK_RAIN;
          break;
        case 'x':
          pdebug = 1;
          break;
        case 'X':
          EXTREM = 1;
          break;
        case 'z':
          resetws = 1;
          break;
        case 'h':
        default:
          show_usage();
          exit (EXIT_SUCCESS);
        }
    }

    if (VIEWMASK == 0)
      VIEWMASK = MASK_ALL;

    if (resetws && strlen(BACKUPFILE) == 0)
    {
       fprintf(stderr, gettext("To reset station log buffer you must to select a backup file with -b option\n"));
       exit (EXIT_FAILURE);
    }

  /* When backup a file, we want all data */
  if (strlen(BACKUPFILE))
    LASTN = 0;
  else if (LASTN >= 4080 || LASTN < 0)
    {
       fprintf(stderr, gettext("You cannot select more than 4079 observations in -l option\n"));
       exit (EXIT_FAILURE);
    }


  /* read calibration points, if any */
  if (strlen(CALFILE))
  {
     if (read_cal_points(&CALWWSR, CALFILE))
     {
        fprintf(stderr, gettext("Problems when reading calibration values from file\n"));
        return 1;
     }

  }
  else
  {
     NOCAL = 1;
     for (j = 0; j < CALWWSR.npf; j++)
       clean_cal_pol(CALWWSR.pf[j].c);
  }

#ifdef WITH_DB
  if (FROMDB  && FROMFILE)
  {
     fprintf(stderr, gettext("Input data is from DB, backup file or device. Cannot join -i and -d options\n"));
     return 1; 
  }

  if (FROMDB && strlen(DBFILE) == 0)
  {
     fprintf(stderr, gettext("You must supply data base filename with '-D' option if you want get data from it\n"));
     return 1; 
  }

  if (KILLDATA && (FROMDB == 0 || strlen(DBFILE) == 0))
  {
     fprintf(stderr, gettext("You must supply data base filename with '-D' option and use '-b' with '-K' option\n"));
     return 1; 
  }

  if (strlen(DBFILE) && FROMDB == 0 && TIMESET == 0)
  {
     fprintf(stderr, gettext("To store data in a DB file, you must include '-U gmt_offset' option\n"));
     return 1; 
  }

  if (INTERP && CLIMAT)
  {
     fprintf(stderr, gettext("Options '-T and -S' cannot use simultaneously\n"));
     return 1; 
  }

  if (CLIMAT && (TEND == 0 || TBEGIN== 0))
  {
     fprintf(stderr, gettext("With -S you have to use both -B and -E options\n"));
     return 1; 
  }
#endif

  return 0;
}


/*!
  \fn int write_backup_file(char *name, size_t nbytes)
  \brief write the station memory data in a file, in binary form

  If problems returns a non zero value
*/
int write_backup_file(char *name, size_t nbytes)
{
   size_t nw;
   FILE *f;
   
   if ((f = fopen(name,"w")) == NULL)
   {
      fprintf(stderr,gettext("Cannot open %s backup file\n"), name);
      return 1; 
   }

  if ((nw = fwrite(LOGBUF, 1, nbytes, f)) != nbytes)
    {
      fprintf(stderr, gettext("problem writing data. Readed %u. Wrote %u\n"), (unsigned int)nbytes, (unsigned int)nw);
      fclose(f);
      return 1;
    }
 
  fclose(f);
  return 0;
}
