/* $Id: wwsr_usb.c 19 2009-02-01 19:07:45Z gbv $ */
/***************************************************************************
 *                                                                         *
 *   wwsr - Wireless Weather Station Reader                                *
 *   2007 dec 19, Michael Pendec (michael.pendec@gmail.com)                *
 *   Version 0.1                                                           *
 *   2008 jan 24 Svend Skafte (svend@skafte.net)                           *
 *   Modifications for different firmware version(?)                       *
 *   2009 Jan 21 Guillermo Ballester gbv@oxixares.com                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*! \file wwsr_usb.c
   \brief This file includes all usb routines for wwsr
*/
#include "wwsr.h"

int timeout = 1000;

/*************************************************************************************/
/* close device */
/*************************************************************************************/
void _close_readw()
{
  ret = usb_release_interface(devh, 0);
  if (ret != 0)
    fprintf( stderr, gettext("could not release interface: %d\n"), ret);
  ret = usb_close(devh);
  if (ret != 0)
    fprintf( stderr, gettext("Error closing interface: %d\n"), ret);
}

/*************************************************************************************/
/* Find device matching vendor and product */
/*************************************************************************************/
struct usb_device *find_device(int vendor, int product)
  {
    struct usb_bus *bus;

    for (bus = usb_get_busses(); bus; bus = bus->next)
      {
        struct usb_device *dev;

        for (dev = bus->devices; dev; dev = dev->next)
          {
            if (dev->descriptor.idVendor == vendor
                && dev->descriptor.idProduct == product)
              return dev;
          }
      }
    return NULL;
  }

/*************************************************************************************/
/* Open Device for read */
/*************************************************************************************/
void _open_readw()
{
  struct usb_device *dev;
  int vendor, product;
#if 0
  usb_urb *isourb;
  struct timeval isotv;
  char isobuf[32768];
#endif

  usb_init();
//    usb_set_debug(0);
  usb_find_busses();
  usb_find_devices();

  vendor = 0x1941;
  product = 0x8021;

  dev = find_device(vendor, product);
  assert(dev);
  devh = usb_open(dev);
  assert(devh);
  signal(SIGTERM, _close_readw);
  ret = usb_get_driver_np(devh, 0, usbmsg, sizeof(usbmsg));
  if (ret == 0)
    {
      // printf("interface 0 already claimed by driver \"%s\", attempting to detach it\n", usbmsg);
      ret = usb_detach_kernel_driver_np(devh, 0);
      // printf("usb_detach_kernel_driver_np returned %d\n", ret);
    }
  ret = usb_claim_interface(devh, 0);
  if (ret != 0)
    {
      fprintf( stderr, gettext("Could not open usb device, errorcode - %d\n"), ret);
      exit(1);
    }
  ret = usb_set_altinterface(devh, 0);
  assert(ret >= 0);
}

/*************************************************************************************/
/* Init for read */
/*************************************************************************************/
void _init_wread()
{
  char tbuf[1000];
  ret = usb_get_descriptor(devh, 1, 0, tbuf, 0x12);
  // usleep(14*1000);
  ret = usb_get_descriptor(devh, 2, 0, tbuf, 0x09);
  // usleep(10*1000);
  ret = usb_get_descriptor(devh, 2, 0, tbuf, 0x22);
  // usleep(22*1000);
  ret = usb_release_interface(devh, 0);
  if (ret != 0) fprintf(stderr, gettext("failed to release interface before set_configuration: %d\n"), ret);
  ret = usb_set_configuration(devh, 1);
  ret = usb_claim_interface(devh, 0);
  if (ret != 0) fprintf(stderr, gettext("claim after set_configuration failed with error %d\n"), ret);
  ret = usb_set_altinterface(devh, 0);
  // usleep(22*1000);
  ret = usb_control_msg(devh, USB_TYPE_CLASS + USB_RECIP_INTERFACE, 0x000000a, 0x0000000, 0x0000000, tbuf, 0x0000000, timeout);
  // usleep(4*1000);
  ret = usb_get_descriptor(devh, 0x22, 0, tbuf, 0x74);
}

/*************************************************************************************/
/* send usb message */
/*************************************************************************************/
void _send_usb_msg( char msg1[1],char msg2[1],char msg3[1],char msg4[1],char msg5[1],char msg6[1],char msg7[1],char msg8[1] )
{
  char tbuf[1000];
  tbuf[0] = msg1[0];
  tbuf[1] = msg2[0];
  tbuf[2] = msg3[0];
  tbuf[3] = msg4[0];
  tbuf[4] = msg5[0];
  tbuf[5] = msg6[0];
  tbuf[6] = msg7[0];
  tbuf[7] = msg8[0];
  //print_bytes(tbuf, 8);
  //printf(" - - - \n");
  ret = usb_control_msg(devh, USB_TYPE_CLASS + USB_RECIP_INTERFACE, 0x0000009, 0x0000200, 0x0000000, tbuf, 0x0000008, timeout);
  //ret = usb_control_msg(devh, USB_TYPE_CLASS + USB_RECIP_INTERFACE, 9, 0x200, 0, tbuf, 8, 1000);
  //usleep(28*1000);
}

/*************************************************************************************/
/* read usb message */
/*************************************************************************************/
void _read_usb_msg(char *buffer)
{
  char tbuf[1000];
  usb_interrupt_read(devh, 0x81, tbuf, 0x20, timeout);
  memcpy(buffer, tbuf, 0x20);
  // usleep(82*1000);
}


/*************************************************************************************/
/* reset ws history                                                                  */
/*************************************************************************************/
void reset_ws(void)
{
  char ec='n';

  printf(gettext(" Resetting WetterStation history\n"));
  printf(gettext("Sure you want to reset wetter station (y/N)?"));
  fflush(stdin);
  scanf("%c",&ec);
  if ( (ec == 'y') || (ec == 'Y') || (ec == 's') || (ec == 'S'))
    {
      _send_usb_msg("\xa0","\x00","\x00","\x20","\xa0","\x00","\x00","\x20");
      _send_usb_msg("\x55","\x55","\xaa","\xff","\xff","\xff","\xff","\xff");
      usleep(28*1000);
      _send_usb_msg("\xff","\xff","\xff","\xff","\xff","\xff","\xff","\xff");
      usleep(28*1000);
      _send_usb_msg("\x05","\x20","\x01","\x38","\x11","\x00","\x00","\x00");
      usleep(28*1000);
      _send_usb_msg("\x00","\x00","\xaa","\x00","\x00","\x00","\x20","\x3e");
      usleep(28*1000);
    }
  else
    {
      printf(gettext(" Aborted reset of history buffer\n"));
    }
}
