/* $Id: wwsr_utils.c 38 2009-04-25 21:35:47Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file wwsr_utils.c
  \brief some util routines for wwsr
*/
#include "wwsr.h"

uint16_t read_uint16(uint8_t *bf)
{
  return ((uint16_t)bf[0] + (uint16_t)bf[1] * 256);
}

int16_t read_sint16(uint8_t *bf)
{
  if (bf[1] & 0x80)
    return -((uint16_t)bf[0] + (uint16_t)(bf[1] & 0x7f) * 256);
  else
    return read_uint16(bf);
}


void print_bytes(char *bytes, int len)
{
  int i;
  if (len > 0)
    {
      for (i=0; i<len; i++)
        {
          printf("%02x ", (int)((unsigned char)bytes[i]));
        }
      // printf("\"");
    }
}


/*!
 \fn size_t get_main_block(char *buf)
 \brief get first 256 bytes, the main block
*/
size_t get_main_block(char *buf)
{

  /* case of reading from binary file */
  if (FROMFILE)
    {
      fseek(F, 0, SEEK_SET); /* move to the begining */
      return fread(buf, 1, 256, F);
    }

  /* get 256 bytes block and set them in buf2 */
  /* get first 32 bytes from first block */
  _send_usb_msg("\xa1","\x00","\x00","\x20","\xa1","\x00","\x00","\x20");
  _read_usb_msg(buf);
  if ( ( pdebug == 1) )
    {
      printf("000-031: ");
      print_bytes(buf, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x20","\x20","\xa1","\x00","\x20","\x20");
  _read_usb_msg(buf + 32);
  if ( ( pdebug == 1) )
    {
      printf("032-063: ");
      print_bytes(buf + 32, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x40","\x20","\xa1","\x00","\x40","\x20");
  _read_usb_msg(buf + 64);
  if ( ( pdebug == 1) )
    {
      printf("064-095: ");
      print_bytes(buf + 64, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x60","\x20","\xa1","\x00","\x60","\x20");
  _read_usb_msg(buf + 96);
  if ( ( pdebug == 1) )
    {
      printf("096-123: ");
      print_bytes(buf + 96, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\x80","\x20","\xa1","\x00","\x80","\x20");
  _read_usb_msg(buf + 128);
  if ( ( pdebug == 1) )
    {
      printf("124-159: ");
      print_bytes(buf + 128, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\xa0","\x20","\xa1","\x00","\xa0","\x20");
  _read_usb_msg(buf + 160);
  if ( ( pdebug == 1) )
    {
      printf("160-191: ");
      print_bytes(buf + 160, 32);
      printf("\n");
    }

  /* Next 32 bytes */
  _send_usb_msg("\xa1","\x00","\xc0","\x20","\xa1","\x00","\xc0","\x20");
  _read_usb_msg(buf + 192);
  if ( ( pdebug == 1) )
    {
      printf("192-223: ");
      print_bytes(buf + 192, 32);
      printf("\n");
    }

  /* Latest 32 bytes */
  _send_usb_msg("\xa1","\x00","\xe0","\x20","\xa1","\x00","\xe0","\x20");
  _read_usb_msg(buf + 224);
  if ( ( pdebug == 1) )
    {
      printf("224-255: ");
      print_bytes(buf + 224, 32);
      printf("\n");
    }
  return 256;
}

void print_obs_verbose(FILE *f, struct obs_data *o)
{
  struct tm tim;
  char aux[32];

  if ( (TBEGIN == 0 || o->datime >= TBEGIN) &&
       (TEND == 0 || o->datime <= TEND))
    {
      memset(&tim, 0, sizeof(struct tm));
      gmtime_r(&(o->datime), &tim);
      strftime(aux, 32, "%Y-%m-%d %H:%M UTC\n", &tim);
      fprintf(f, gettext("Date    : %s\n"), aux);
      fprintf(f, gettext("Interval: %u minutes\n"), (unsigned int) o->interval);
      if (VIEWMASK & MASK_HUM_INT)
        fprintf(f, gettext("Hum int : %u %%\n"), (unsigned int) o->hum_int);
      if (VIEWMASK & MASK_TEMP_INT)
        fprintf(f, gettext("Tem int : %.1f C\n"), (float)(o->temp_int) * 0.1);
      if (VIEWMASK & MASK_HUM_EXT)
        fprintf(f, gettext("Hum ext : %u %%\n"), (unsigned int)o->hum_out);
      if (VIEWMASK & MASK_TEMP_EXT)
        fprintf(f, gettext("Tem ext : %.1f C\n"), (float)(o->temp_out) * 0.1);
      if (VIEWMASK & MASK_PRES)
        fprintf(f, gettext("Pressure: %.1f HPa\n"), (float)(o->press) * 0.1);
      if ((VIEWMASK & MASK_WIND) == MASK_WIND)
        {
          fprintf(f, gettext("Wind    : %.1f m/s\n"), (float)(o->wspd) * 0.1);
          if (o->wdir > 16)
            o->wdir = 16;
          fprintf(f, gettext("Wdir    : %s \n"), wd[o->wdir]);
          fprintf(f, gettext("Gust    : %.1f m/s\n"), (float)(o->wgust) * 0.1);
        }
      if (VIEWMASK & MASK_RAIN)
        fprintf(f, gettext("Ac. prec: %.1f mm\n"), (float)(o->prec) * 0.3);
      fprintf(f, "#-------------------\n");
    }
}

void print_obs(FILE *f, struct obs_data *o, char *sep)
{
  struct tm tim;
  char aux[32], *c, lin[256];

  /* check if valid data */
  if ( o->mask == 0xFF || (o->hum_int == 0 && o->interval == 0))
    return;

  /* check about date */
  if ( (TBEGIN == 0 || o->datime >= TBEGIN) &&
       (TEND == 0 || o->datime <= TEND))
    {
      memset(&tim, 0, sizeof(struct tm));
      gmtime_r(&(o->datime), &tim);
      strftime(aux, 32, "%Y%m%d%H%M", &tim);
      c = lin + sprintf(lin, "%s%s", aux, sep);
      if (VIEWMASK & MASK_INTERV)
        c += sprintf(c, "%u%s", (unsigned int) o->interval, sep);
      if (VIEWMASK & MASK_HUM_INT)
        c += sprintf(c, "%.0lf%s", value((double) o->hum_int, &CALhi), sep);
      if (VIEWMASK & MASK_TEMP_INT)
        c += sprintf(c, "%.1lf%s", value((double)(o->temp_int) * 0.1, &CALti), sep);
      if (VIEWMASK & MASK_HUM_EXT)
        c += sprintf(c, "%.0lf%s", value((double) o->hum_out, &CALho), sep);
      if (VIEWMASK & MASK_TEMP_EXT)
        c += sprintf(c, "%.1lf%s", value((double)(o->temp_out) * 0.1, &CALto), sep);
      if (VIEWMASK & MASK_PRES)
        c += sprintf(c, "%.1lf%s", value((double)(o->press) * 0.1, &CALp), sep);
      if ((VIEWMASK & MASK_WIND) == MASK_WIND)
        {
          c += sprintf(c, "%.1lf%s", value((double)(o->wspd) * 0.1, &CALw), sep);
          if (o->wdir > 16)
            o->wdir = 16;
          c += sprintf(c, "%s%s", wd[o->wdir], sep);
          c += sprintf(c, "%.1lf%s", value((double)(o->wgust) * 0.1, &CALw), sep);
        }
      if (VIEWMASK & MASK_RAIN)
        c += sprintf(c, "%.1lf%s", value((double)(o->prec) * 0.3, &CALr), sep);
      *(c - 1) = '\n';
      fprintf( f, "%s", lin);
    }
}

/* this version do not check for time */
void print_obs_nores(FILE *f, struct obs_data *o, char *sep)
{
  struct tm tim;
  char aux[32], *c, lin[256];

  /* check if valid data */
  if ( o->mask == 0xFF || (o->hum_int == 0 && o->interval == 0))
    return;

  memset(&tim, 0, sizeof(struct tm));
  gmtime_r(&(o->datime), &tim);
  strftime(aux, 32, "%Y%m%d%H%M", &tim);
  c = lin + sprintf(lin, "%s%s", aux, sep);
  if (VIEWMASK & MASK_INTERV)
    c += sprintf(c, "%u%s", (unsigned int) o->interval, sep);
  if (VIEWMASK & MASK_HUM_INT)
    c += sprintf(c, "%.0lf%s", value((double) o->hum_int, &CALhi), sep);
  if (VIEWMASK & MASK_TEMP_INT)
    c += sprintf(c, "%.1lf%s", value((double)(o->temp_int) * 0.1, &CALti), sep);
  if (VIEWMASK & MASK_HUM_EXT)
    c += sprintf(c, "%.0lf%s", value((double) o->hum_out, &CALho), sep);
  if (VIEWMASK & MASK_TEMP_EXT)
    c += sprintf(c, "%.1lf%s", value((double)(o->temp_out) * 0.1, &CALto), sep);
  if (VIEWMASK & MASK_PRES)
    c += sprintf(c, "%.1lf%s", value((double)(o->press) * 0.1, &CALp), sep);
  if ((VIEWMASK & MASK_WIND) == MASK_WIND)
    {
      c += sprintf(c, "%.1lf%s", value((double)(o->wspd) * 0.1, &CALw), sep);
      if (o->wdir > 16)
        o->wdir = 16;
      c += sprintf(c, "%s%s", wd[o->wdir], sep);
      c += sprintf(c, "%.1lf%s", value((double)(o->wgust) * 0.1, &CALw), sep);
    }
  if (VIEWMASK & MASK_RAIN)
    c += sprintf(c, "%.1lf%s", value((double)(o->prec) * 0.3, &CALr), sep);
  *(c - 1) = '\n';
  fprintf( f, "%s", lin);
}

/* this version do not check for time */
void print_real_data(FILE *f, struct real_data *r,const char *sep)
{
  struct tm tim;
  char aux[32], *c, lin[256];

  memset(&tim, 0, sizeof(struct tm));
  gmtime_r(&(r->datime), &tim);
  strftime(aux, 32, "%Y%m%d%H%M", &tim);
  c = lin + sprintf(lin, "%s%s", aux, sep);
  c += sprintf(c, "%.0lf%s",  r->interval, sep);
  c += sprintf(c, "%.0lf%s", r->hum_int, sep);
  c += sprintf(c, "%.1lf%s", r->temp_int, sep);
  c += sprintf(c, "%.0lf%s", r->hum_out, sep);
  c += sprintf(c, "%.1lf%s", r->temp_out, sep);
  c += sprintf(c, "%.1lf%s", r->press, sep);
  c += sprintf(c, "%.1lf%s", r->wspd, sep);
  c += sprintf(c, "%.0lf%s", r->wdir, sep);
  c += sprintf(c, "%.1lf%s", r->wgust, sep);
  c += sprintf(c, "%.1lf%s", r->prec, sep);
  *(c - 1) = '\n';
  fprintf( f, "%s", lin);
}


uint8_t bcdval( uint8_t a)
{
  return ((a >> 4) * 10) + (a & 0x0f);
}

int buf2obs (struct obs_data *o, unsigned char *buf)
{
  o->interval = buf[0];
  o->hum_int = buf[1];
  o->temp_int = read_sint16(&buf[2]);
  /* o->temp_int = buf[2] + buf[3] * 256; */
  o->hum_out = buf[4];
  o->temp_out = read_sint16(&buf[5]);
  /* o->temp_out = buf[5] + buf[6] * 256; */
  o->press =  buf[7] + buf[8] * 256;
  o->wspd = buf[9];
  o->wgust = buf[10];
  o->wdir = buf[12];
  o->prec = buf[13] + buf[14] * 256;
  o->mask = buf[15];
  memcpy(&o->buf[0], buf, 16);
  return 0;
}

void read_date(struct tm *tim, uint8_t *bf)
{
  time_t t;
  memset(tim, 0, sizeof(struct tm));
  tim->tm_year = (int)100 + bcdval(bf[0]);
  tim->tm_mon = bcdval(bf[1]) - 1;
  tim->tm_mday = bcdval(bf[2]);
  tim->tm_hour = bcdval(bf[3]);
  tim->tm_min = bcdval(bf[4]);
  tim->tm_sec = 0;
  t = timegm(tim) - GMTOFF;
}

void read_extremes(uint8_t *bf)
{
  /* first the values */
  hin_max.dat = value((double) bf[0x62], &CALhi);
  hin_min.dat = value((double)bf[0x63], &CALhi);
  hout_max.dat = value((double)bf[0x64], &CALho);
  hout_min.dat = value((double)bf[0x65], &CALho);
  tin_max.dat = value((double) read_sint16(bf + 0x66) * 0.1, &CALti);
  tin_min.dat = value((double) read_sint16(bf + 0x68) * 0.1, &CALti);
  tout_max.dat = value((double) read_sint16(bf + 0x6a) * 0.1, &CALto);
  tout_min.dat = value((double) read_sint16(bf + 0x6c) * 0.1, &CALto);
  chill_max.dat = (double) read_sint16(bf + 0x6e) * 0.1;
  chill_min.dat = (double) read_sint16(bf + 0x70) * 0.1;
  dwp_max.dat = (double) read_sint16(bf + 0x72) * 0.1;
  dwp_min.dat = (double) read_sint16(bf + 0x74) * 0.1;
  apres_max.dat = value((double) read_uint16(bf + 0x76) * 0.1, &CALp);
  apres_min.dat = value((double) read_uint16(bf + 0x78) * 0.1, &CALp);
  rpres_max.dat = (double) read_uint16(bf + 0x7a) * 0.1;
  rpres_min.dat = (double) read_uint16(bf + 0x7c) * 0.1;
  wsp_max.dat = value((double) read_uint16(bf + 0x7e) * 0.1, &CALw);
  gust_max.dat = value((double) read_uint16(bf + 0x80) * 0.1, &CALw);
  rh_max.dat = value((double) read_uint16(bf + 0x82) * 0.1, &CALr);
  rd_max.dat = value((double) read_uint16(bf + 0x84) * 0.1, &CALr);
  rw_max.dat = value((double) read_uint16(bf + 0x86) * 0.1, &CALr);
  rm_max.dat = value((double) read_uint16(bf + 0x88) * 0.1, &CALr);
  rt_max.dat = value((double) read_uint16(bf + 0x8a) * 0.1, &CALr);
  read_date( &hin_max.tim, bf + 0x8d);
  read_date( &hin_min.tim, bf + 0x92);
  read_date( &hout_max.tim, bf + 0x97);
  read_date( &hout_min.tim, bf + 0x9c);
  read_date( &tin_max.tim, bf + 0xa1);
  read_date( &tin_min.tim, bf + 0xa6);
  read_date( &tout_max.tim, bf + 0xab);
  read_date( &tout_min.tim, bf + 0xb0);
  read_date( &chill_max.tim, bf + 0xb5);
  read_date( &chill_min.tim, bf + 0xba);
  read_date( &dwp_max.tim, bf + 0xbf);
  read_date( &dwp_min.tim, bf + 0xc4);
  read_date( &apres_max.tim, bf + 0xc9);
  read_date( &apres_min.tim, bf + 0xce);
  read_date( &rpres_max.tim, bf + 0xd3);
  read_date( &rpres_min.tim, bf + 0xd8);
  read_date( &wsp_max.tim, bf + 0xdd);
  read_date( &gust_max.tim, bf + 0xe2);
  read_date( &rh_max.tim, bf + 0xe7);
  read_date( &rd_max.tim, bf + 0xec);
  read_date( &rw_max.tim, bf + 0xf1);
  read_date( &rm_max.tim, bf + 0xf6);
  read_date( &rt_max.tim, bf + 0xfb);
}

void print_extreme(FILE *f, struct extrem_data *d, char *name, char *unit)
{
  char aux[32];
  time_t ti;

  ti = timegm(&d->tim);
  if ( (TBEGIN == 0 || ti >= TBEGIN) &&
       (TEND == 0 || ti <= TEND))
    {
      strftime(aux, 32, "%Y-%m-%d %H:%M", &d->tim);
      fprintf(f, "%s:\t %6.1f %s \t%s\n", name, d->dat, unit, aux);
    }
  else
    {
      fprintf(f, gettext("%s:\t Out of selected date interval\n"), name);
    }
}

void print_all_extremes(FILE *f)
{
  if (VIEWMASK & MASK_HUM_INT)
    {
      print_extreme(f, &hin_max,  gettext("Hum. int. max. "),"%");
      print_extreme(f, &hin_min,  gettext("Hum. int. min. "),"%");
    }
  if (VIEWMASK & MASK_HUM_EXT)
    {
      print_extreme(f, &hout_max, gettext("Hum. out. max. "),"%");
      print_extreme(f, &hout_min, gettext("Hum. out. min. "),"%");
    }
  if (VIEWMASK & MASK_TEMP_INT)
    {
      print_extreme(f, &tin_max,  gettext("Tem. int. max. "),"C");
      print_extreme(f, &tin_min,  gettext("Tem. int. min. "),"C");
    }
  if (VIEWMASK & MASK_TEMP_EXT)
    {
      print_extreme(f, &tout_max, gettext("Tem. out. max. "),"C");
      print_extreme(f, &tout_min, gettext("Tem. out. min. "),"C");
    }
  if ((VIEWMASK & MASK_TEMP_EXT) && (VIEWMASK & MASK_WSPD))
    {
      print_extreme(f, &chill_max, gettext("Wnd. chill max."),"C");
      print_extreme(f, &chill_min, gettext("Wnd. chill min."),"C");
    }
  if ((VIEWMASK & MASK_TEMP_EXT) && (VIEWMASK & MASK_HUM_EXT))
    {
      print_extreme(f, &dwp_max, gettext( "Dew point max. "),"C");
      print_extreme(f, &dwp_min, gettext( "Dew point min. "),"C");
    }
  if (VIEWMASK & MASK_PRES)
    {
      print_extreme(f, &apres_max, gettext("Abs. Pres. max."),"Hpa");
      print_extreme(f, &apres_min, gettext("Abs. Pres. min."),"Hpa");
      print_extreme(f, &rpres_max, gettext("Rel. Pres. max."),"Hpa");
      print_extreme(f, &rpres_min, gettext("Rel. Pres. min."),"Hpa");
    }
  if (VIEWMASK & MASK_WSPD)
    {
      print_extreme(f, &wsp_max,  gettext("Wind speed max."),"m/s");
      print_extreme(f, &gust_max, gettext("Wind gust max. "),"m/s");
    }
  if (VIEWMASK & MASK_RAIN)
    {
      print_extreme(f, &rh_max, gettext("Hour prec. max."), "mm");
      print_extreme(f, &rd_max, gettext("Day. prec. max."), "mm");
      print_extreme(f, &rw_max, gettext("Week prec. max."), "mm");
      print_extreme(f, &rm_max, gettext("Month prec. max"), "mm");
      print_extreme(f, &rt_max, gettext("Total prec. max"), "mm");
    }
}

/*!
  \fn size_t read_station_buf(uint8_t *b, size_t limit)
  \brief Stores a binary image of station memory in BUFLOG

  returns the number of bytes readed
*/
size_t read_station_buf_old( uint8_t *b , size_t limit)
{
  int i;
  uint16_t current, buftemp, inipos;
  char aux[16];


  /*read main block (first 256 bytes) */
  printf(gettext("Reading main control block ... "));
  get_main_block((char *)b);
  printf("Done\n");

  /* current observation position in memory */
  current = read_uint16(b + 30);

  if (LASTN == 0)
    {
      inipos = 0;
      printf(gettext("Reading all the data\n"));
    }
  else
    {
      inipos = current - 0x0100 - LASTN * 16;
      if (inipos > current)
        inipos = 0;
      printf(gettext("Reading latest %d data\n"), LASTN);
    }
  /* read the rest of data */
  for ( buftemp = 0x0100 + inipos, i = 0; buftemp <= current && buftemp <= (limit - 32); buftemp += 32)
    {
      if (FROMFILE)
        {
          fseek(F, buftemp, SEEK_SET);
          if (fread(b + buftemp, 1, 32, F) != 32)
            {
              fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, buftemp);
              return buftemp;
            }
        }
      else
        {
          aux[1] = ( buftemp >>8 & 0xFF ) ;
          aux[2] = buftemp & 0xFF;
          aux[3] = ( buftemp >>8 & 0xFF ) ;
          aux[4] = buftemp & 0xFF;
          _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
          _read_usb_msg((char *)(b + buftemp));
        }
      i += 2;
      if ((i & 15) == 0)
        {
          fprintf(stdout, ".");
          fflush(stdout);
        }
      if ((i & 1023) == 0)
        {
          fprintf(stdout, "\n");
          fflush(stdout);
        }
    };
  fprintf(stdout, "\n");
  return buftemp;
}

/*!
  \fn unsigned int get_pos(uint16_t current, int nobs)
  \brief get the position of a observation
  \param current position of current data
  \param nobs observations before current data
*/
unsigned int get_pos(uint16_t current, int nobs)
{
  int pos;

  pos = (int)current - nobs * 16;
  if (pos < 256)
    pos = MEMSIZE - (256 - pos);
  return pos;
}

/*!
  \fn size_t read_station_buf(uint8_t *b, size_t limit)
  \brief Stores a binary image of station memory in BUFLOG

  returns the number of bytes readed
*/
size_t read_station_buf( uint8_t *b , size_t limit)
{
  int i;
  unsigned int nob, buftemp, inipos, curpos, nr;
  uint16_t current;
  uint8_t bufob[32];
  char aux[16];


  /*read main block (first 256 bytes) */
  printf(gettext("Reading main control block ... "));
  get_main_block((char *)b);
  printf("Done\n");

  /* current observation position in memory */
  current = read_uint16(b + 30);
  /*
     first read current data block, then it is replaced in memory because it may change when reading
     other data
  */
  if (current > 0xFFE0)
    curpos = 0xFFE0;
  else
    curpos = current;

  if (FROMFILE)
    {
      fseek(F, curpos, SEEK_SET);
      if ((nr = fread(bufob, 1, 32, F)) != 32)
        {
          fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, curpos);
        }
    }
  else
    {
      aux[1] = ( curpos >>8 & 0xFF ) ;
      aux[2] = curpos & 0xFF;
      aux[3] = ( curpos >>8 & 0xFF ) ;
      aux[4] = curpos & 0xFF;
      _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
      _read_usb_msg((char *)(bufob));
      nr = 32;
    }


  /* read the rest of data */
  if (LASTN == 0)
    {
      printf(gettext("Reading all the data\n"));
      nob = 4080;
      for ( buftemp = 0x0100, i = 0; buftemp < 0x10000; buftemp += 32)
        {
          if (FROMFILE)
            {
              fseek(F, buftemp, SEEK_SET);
              if (fread(b + buftemp, 1, 32, F) != 32)
                {
                  fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, buftemp);
                  return buftemp;
                }
            }
          else
            {
              aux[1] = ( buftemp >>8 & 0xFF ) ;
              aux[2] = buftemp & 0xFF;
              aux[3] = ( buftemp >>8 & 0xFF ) ;
              aux[4] = buftemp & 0xFF;
              _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
              _read_usb_msg((char *)(b + buftemp));
            }
          i += 2;
          if ((i & 15) == 0)
            {
              fprintf(stdout, ".");
              fflush(stdout);
            }
          if ((i & 1023) == 0)
            {
              fprintf(stdout, "\n");
              fflush(stdout);
            }
        }
      /* Now restore current data block */
      memcpy(b + curpos, bufob, nr);
      return MEMSIZE;
    }
  else
    {
      /* current observation position in memory */
      current = read_uint16(b + 30);

      /* block of 32 bytes */
      inipos = current - 0x0100 - LASTN * 16;
      if (inipos & 0x0010)
        {
          nob = LASTN + 1;
        }
      else
        nob = LASTN;

      printf(gettext("Reading latest %d data\n"), LASTN);

      /* read the rest of data */
      for ( i = nob; i >= 0; i -= 2)
        {
          buftemp = get_pos (current, i);

          if (FROMFILE)
            {
              fseek(F, buftemp, SEEK_SET);
              if (fread(b + buftemp, 1, 32, F) != 32)
                {
                  fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, buftemp);
                  return buftemp;
                }
            }
          else
            {
              aux[1] = ( buftemp >>8 & 0xFF ) ;
              aux[2] = buftemp & 0xFF;
              aux[3] = ( buftemp >>8 & 0xFF ) ;
              aux[4] = buftemp & 0xFF;
              _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
              _read_usb_msg((char *)(b + buftemp));
            }
        }

      /* Now restore current data block */
      memcpy(b + curpos, bufob, nr);
      return (256 + nob  * 32);
    }
}


size_t read_current(uint8_t *b, uint16_t current)
{
  char aux[16];

  if (FROMFILE)
    {
      fseek(F, current, SEEK_SET);
      if (fread(b + current, 1, 32, F) != 32)
        {
          fprintf(stderr, gettext("Problem reading data from file %s at position %u\n"), INPUTFILE, current);
          return 32;
        }
    }
  else
    {
      aux[1] = ( current >> 8 & 0xFF ) ;
      aux[2] = current & 0xFF;
      aux[3] = ( current >> 8 & 0xFF ) ;
      aux[4] = current & 0xFF;
      _send_usb_msg("\xa1", aux+ 1, aux + 2, "\x20", "\xa1", aux + 3, aux + 4,"\x20");
      _read_usb_msg((char *)(b + current));
      /* mark as current observation in mask bit */
      b[current + 15] &= 0x01;
    }
  return 32;
}
